"""
``pywebio_battery`` --- PyWebIO battery
=========================================

*Utilities that help write PyWebIO apps quickly and easily.*

Functions list
-----------------

.. list-table::

   * - Function name
     - Description

   * - `confirm <pywebio_battery.confirm>`
     - Confirmation modal



"""
from .interaction import *
from .web import *
from .interaction import __all__ as interaction_all
from .web import __all__ as web_all

__all__ = interaction_all + web_all

# Set default logging handler to avoid "No handler found" warnings.
import logging

logging.getLogger(__name__).addHandler(logging.NullHandler())
del logging



from pywebio.output import *
from pywebio.input import *
from pywebio.session import *
from pywebio.session import get_current_session
from pywebio.pin import *
from tornado.web import create_signed_value, decode_signed_value

__all__ = ['get_all_query', 'get_query', 'set_localstorage', 'get_localstorage', 'set_cookie', 'get_cookie', 'login']


def get_all_query():
    """Get URL parameter (also known as "query strings" or "URL query parameters") as a dict"""
    query = eval_js("Object.fromEntries(new URLSearchParams(window.location.search))")
    return query


def get_query(name):
    """Get URL parameter value"""
    query = eval_js("new URLSearchParams(window.location.search).get(n)", n=name)
    return query


def set_localstorage(key, value):
    """Save data to user's web browser

    The data is specific to the origin (protocol+domain+port) of the app.
    Different origins use different web browser local storage.

    :param str key: the key you want to create/update.
    :param str value: the value you want to give the key you are creating/updating.

    You can read the value by using :func:`get_localstorage(key)`
    """
    run_js("localStorage.setItem(key, value)", key=key, value=value)


def get_localstorage(key) -> str:
    """Get the key's value in user's web browser local storage"""
    return eval_js("localStorage.getItem(key)", key=key)


def _init_cookie_client():
    session = get_current_session()
    if 'cookie_client_flag' not in session.internal_save:
        session.internal_save['cookie_client_flag'] = True
        # Credit: https://stackoverflow.com/questions/14573223/set-cookie-and-get-cookie-with-javascript
        run_js("""
        window.setCookie = function(name,value,days) {
            var expires = "";
            if (days) {
                var date = new Date();
                date.setTime(date.getTime() + (days*24*60*60*1000));
                expires = "; expires=" + date.toUTCString();
            }
            document.cookie = name + "=" + (value || "")  + expires + "; path=/";
        }
        window.getCookie = function(name) {
            var nameEQ = name + "=";
            var ca = document.cookie.split(';');
            for(var i=0;i < ca.length;i++) {
                var c = ca[i];
                while (c.charAt(0)==' ') c = c.substring(1,c.length);
                if (c.indexOf(nameEQ) == 0) return c.substring(nameEQ.length,c.length);
            }
            return null;
        }
        """)


def set_cookie(key, value, days=7):
    _init_cookie_client()
    run_js("setCookie(key, value, days)", key=key, value=value, days=days)


def get_cookie(key):
    _init_cookie_client()
    return eval_js("getCookie(key)", key=key)


def login(*, basic_auth=None, custom_auth=None, salt=None, expire_days=7):
    """Persistence auth

    You need to provide a function to implement your auth logic and pass it to ``basic_auth`` or ``custom_auth``
    parameter.

    :param callable basic_auth: ``(username, password) -> is_succeed:bool``
    :param callable custom_auth: ``() -> username:str``
    :param str salt: HMAC secret for the signature. It should be a long, random str.
    :param int expire_days: how many days the auth state can keep valid.
       After this time, logged-in users need to log in again.
    :return str: username of the currently logged-in user
    """
    assert bool(basic_auth) != bool(custom_auth), "You can only assign to one of `basic_auth` or `custom_auth`"

    token = get_localstorage('webio-token')  # get token from user's web browser
    # try to decrypt the username from the token
    username = decode_signed_value(salt, 'token', token, max_age_days=expire_days)
    if not token or not username:  # no token or token validation failed
        while True:
            if basic_auth:
                user = input_group('Login', [
                    input("Username", name='username'),
                    input("Password", type=PASSWORD, name='password'),
                ])
                username = user['username']
                ok = basic_auth(username, user['password'])
            else:
                username = ok = custom_auth()

            if ok:
                signed = create_signed_value(salt, 'token', username).decode("utf-8")  # encrypt username to token
                set_localstorage('token', signed)  # set token to user's web browser
                break

    return username



__package__ = 'pywebio_battery'
__description__ = 'Utilities that help write PyWebIO apps quickly and easily.'
__url__ = 'https://pywebio.readthedocs.io/en/latest/battery.html'
__version__ = "0.2.0"
__version_info__ = (0, 2, 0, 0)
__author__ = 'WeiminWang'
__author_email__ = 'wang0.618@qq.com'
__license__ = 'MIT'



from pywebio.output import *
from pywebio.input import *
from pywebio.session import *
from pywebio.pin import *
from pywebio.utils import random_str
from pywebio.output import Output
import io
from functools import partial
import subprocess

__all__ = ['confirm', 'popup_input', 'redirect_stdout', 'run_shell', 'put_logbox', 'logbox_append']


def confirm(title, content=None, *, timeout=None):
    """Show a confirmation modal.

    :param str title: Model title.
    :param list/put_xxx() content: Model content.
    :param None/float timeout: Seconds for operation time out.
    :return: Return `True` when the "CONFIRM" button is clicked,
        return `False` when the "CANCEL" button is clicked,
        return `None` when a timeout is given and the operation times out.
    """
    if content is None:
        content = []
    if not isinstance(content, list):
        content = [content]
    action_name = random_str(10)
    content.append(put_actions(action_name, buttons=[
        {'label': 'CONFIRM', 'value': True},
        {'label': 'CANCEL', 'value': False, 'color': 'danger'},
    ]))
    popup(title=title, content=content, closable=False)
    result = pin_wait_change(action_name, timeout=timeout)
    if result:
        result = result['value']
    close_popup()
    return result


def popup_input(pins, title='Please fill out the form below'):
    """Show a form in popup window.

    :param list pins: pin output list.
    :param str title: model title.
    :return: return the form value as dict, return None when user cancel the form.
    """
    if not isinstance(pins, list):
        pins = [pins]

    pin_names = [
        p.spec['input']['name']
        for p in pins
    ]
    action_name = 'action_' + random_str(10)
    pins.append(put_actions(action_name, buttons=[
        {'label': 'Submit', 'value': True},
        {'label': 'Cancel', 'value': False, 'color': 'danger'},
    ]))
    popup(title=title, content=pins, closable=False)

    change_info = pin_wait_change(action_name)
    result = None
    if change_info['name'] == action_name and change_info['value']:
        result = {name: pin[name] for name in pin_names}
    close_popup()
    return result


def redirect_stdout(output_func=partial(put_text, inline=True)):
    """Context manager for temporarily redirecting stdout to pywebio.

    ::

        with redirect_stdout():
            print("Hello world.")
    """
    from contextlib import redirect_stdout

    class WebIO(io.IOBase):
        def write(self, content):
            output_func(content)

    return redirect_stdout(WebIO())


def run_shell(cmd, output_func=partial(put_text, inline=True)):
    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    while True:
        out = process.stdout.readline()
        if out:
            output_func(out.decode('utf8'))

        if not out and process.poll() is not None:
            break


def put_logbox(name, height=None) -> Output:
    """Output a logbox widget

    :param str name:
    :param int height: the height of the widget in pixel
    """
    html = '<pre><code id="webio-logbox-%s"></code></pre>' % name
    return put_html(html).style('height:%spx' % height if height else '')


def logbox_append(name, text):
    """Append text to a logbox widget"""
    run_js('$("#webio-logbox-%s").append(text)' % name, text=str(text))



import os

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(here, 'pywebio_battery', '__version__.py'), encoding='utf8') as f:
    exec(f.read(), about)

with open('README.md', encoding='utf8') as f:
    readme = f.read()


setup(
    name=about['__package__'],
    version=about['__version__'],
    description=about['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    license=about['__license__'],
    python_requires=">=3.5.2",
    packages=['pywebio_battery'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    install_requires=[
        'pywebio',
    ],
    project_urls={
        'Documentation': about['__url__'],
        'Source': 'https://github.com/wang0618/pywebio-battery',
    },
)
